using System;
using System.Collections.Generic;
using System.Text;
using System.Data;
using MySql.Data.MySqlClient;
using System.Collections;
using System.Windows.Forms;
using System.Globalization;


namespace books
{
    /// <summary>
    /// Wraps calls to the users database
    /// </summary>
    public class DbWrapper
    {
        public MySqlConnection sqlConn;
        private string connStr;
        private bool isConnected;


        /// <summary>
        /// Creates a new database wrapper object that wraps around
        /// the users table.
        /// </summary>
        /// <param name="svr">The name of the server</param>
        /// <param name="db">The database catalog to use</param>
        /// <param name="user">The user name</param>
        /// <param name="pass">The user password</param>
        public DbWrapper(string svr, string db, string user, string pass)
        {
            this.connStr = "Server=" + svr + ";Database=" + db + ";Uid=" + user + ";Pwd=" + pass + "; Allow Zero Datetime=True;";

            try
            {
                sqlConn = new MySqlConnection(this.connStr);
            }
            catch (Exception excp)
            {
                Exception myExcp = new Exception("Error connecting you to " +
                    "the my sql server. Internal error message: " + excp.Message, excp);
                throw myExcp;
            }

            this.isConnected = false;
        }

        /// <summary>
        /// Creates a new database wrapper object that wraps around
        /// the users table.
        /// </summary>
        /// <param name="connStr">A connection string to provide to connect
        /// to the database</param>
        public DbWrapper(string connStr)
        {
            this.connStr = connStr;

            try
            {
                sqlConn = new MySqlConnection(this.connStr);
            }
            catch (Exception excp)
            {
                Exception myExcp = new Exception("Error connecting you to " +
                    "the my sql server. Error: " + excp.Message, excp);

                throw myExcp;
            }

            this.isConnected = false;
        }

        /// <summary>
        /// Opens the connection to the SQL database.
        /// </summary>
        public void Connect()
        {
            bool success = true;

            if (this.isConnected == false)
            {
                try
                {
                    this.sqlConn.Open();
                }
                catch (Exception excp)
                {
                    this.isConnected = false;
                    success = false;
                    Exception myException = new Exception("Error opening connection" +
                        " to the sql server. Error: " + excp.Message, excp);

                    throw myException;
                }

                if (success)
                {
                    string pre_Query = "SET NAMES 'utf8'";
                    MySqlCommand names = new MySqlCommand(pre_Query, this.sqlConn);
                    names.ExecuteNonQuery();
                    this.isConnected = true;
                }
            }
        }

        /// <summary>
        /// Closes the connection to the sql connection.
        /// </summary>
        public void Disconnect()
        {
            if (this.isConnected)
            {
                this.sqlConn.Close();
            }
        }

        /// <summary>
        /// Gets the current state (boolean) of the connection.
        /// True for open, false for closed.
        /// </summary>
        public bool IsConnected
        {
            get
            {
                return this.isConnected;
            }
        }

        /// <summary>
        /// Adds a user into the database
        /// </summary>
        /// <param name="username">The user login</param>
        /// <param name="password">The user password</param>




        public DataTable list()
        {
            DataSet models = new DataSet();
            DataTable returnValue = new DataTable();

            string Query = "SELECT SQL_CACHE b.AUTHORLAST,b.AUTHORFIRST,b.BOOKNAME,b.SPCODE,year(e.date) as year,e.bookcode from book b ,expences e where b.spcode = e.bookcode group by e.bookcode ORDER BY b.AUTHORLAST ASC; ";

            MySqlCommand custList = new MySqlCommand(Query, this.sqlConn);

            try
            {
                custList.ExecuteNonQuery();

                MySqlDataReader myReader = custList.ExecuteReader();

                DataReaderAdapter dar = new DataReaderAdapter();
                dar.FillFromReader(returnValue, myReader);





                myReader.Close();
            }
            catch (Exception excp)
            {
                Exception myExcp = new Exception("Could Retrieve the Model List " +
                    excp.Message, excp);
                throw (myExcp);
            }

            return returnValue;
        }




        public DataTable authors_list()
        {
            DataSet users = new DataSet();
            DataTable returnValue = new DataTable();

           // string Query = "Select distinct cust_id,cust_surname as ,cust_area as ,cust_email1 as Email,cust_phone1 as ,cust_cell as  from customers where cust_deleted = 0 and cust_user_id = '" + Program.user_id + "' ";
            string Query = "SELECT SQL_CACHE b.AUTHORLAST ,b.SPCODE,e.bookcode from books.book b ,books.expences e where b.spcode = e.bookcode group by e.bookcode;";

            MySqlCommand UserList = new MySqlCommand(Query, this.sqlConn);

            try
            {
                UserList.ExecuteNonQuery();

                MySqlDataReader myReader = UserList.ExecuteReader();

                DataReaderAdapter dar = new DataReaderAdapter();
                dar.FillFromReader(returnValue, myReader);





                myReader.Close();
            }
            catch (Exception excp)
            {
                Exception myExcp = new Exception("Could Retrieve the User List " +
                    excp.Message, excp);
                throw (myExcp);
            }

            return returnValue;
        }





        public DataTable search_books(string surname,string bookname)
        {
            DataTable returnValue = new DataTable();
            string Query = "SELECT SQL_CACHE b.AUTHORLAST, b.AUTHORFIRST , b.BOOKNAME, b.SPCODE, year(e.Date) as year,e.Bookcode as bookcode from books.book b ,books.expences e where b.Spcode = e.Bookcode and b.Authorlast like '%" + surname + "%' and b.Bookname like '%" + bookname + "%' group by e.Bookcode;";

            MySqlCommand order_data = new MySqlCommand(Query, this.sqlConn);

            try
            {
                order_data.ExecuteNonQuery();

                MySqlDataReader myReader = order_data.ExecuteReader();

                DataReaderAdapter dar = new DataReaderAdapter();
                dar.FillFromReader(returnValue, myReader);

                myReader.Close();
            }
            catch (Exception excp)
            {
                Exception myExcp = new Exception("Could Retrieve the User List " +
                    excp.Message, excp);
                throw (myExcp);
            }

            return returnValue;
        }
        public DataTable search_bydate(string authorlast, string bookname, string startDate, string endDate)
        {
            DataTable returnValue = new DataTable();
            DataTable added = new DataTable();
            string Query;
            if ( authorlast == "" && bookname == "" )
            {
               //Query = "SELECT  CONCAT( b.Authorlast, ' ', b.Authorfirst) as Authorname, b.Bookname as Bookname, e.Spcode as espcode, e.Spauthor, e.Bookcode, e.Year, e.Bookname, e.Auname, DATE_FORMAT(e.Date, '%d/%m/%Y') as Date, e.Tim, e.Who, e.Copies, e.Papers, e.S1, e.S2, e.S3, e.S4, e.S5, e.S6, e.S7, e.S8, e.S9, e.Sum, e.Mem1  from  books.book b, books.expences e WHERE e.Date BETWEEN '" + startDate + "' and '" + endDate + "' and b.Spcode = e.Bookcode order by e.Bookcode desc;";
                Query = "SELECT  CONCAT( b.Authorlast, ' ', b.Authorfirst) as Authorname, b.Bookname as Bookname, e.Spcode as espcode, e.Bookcode, e.Year, e.Bookname, e.Auname, DATE_FORMAT(e.Date, '%d/%m/%Y') as Date, e.Tim, e.Who, e.Copies, e.Papers, e.S1, e.S2, e.S3, e.S4, e.S5, e.S6, e.S7, e.S8, e.S9, e.Sum, e.Mem1  from  books.book b, books.expences e WHERE e.Date BETWEEN '" + startDate + "' and '" + endDate + "' and b.Spcode = e.Bookcode order by e.Bookcode desc;";
            }
            else
            {
                //Query = "SELECT  CONCAT( b.Authorlast, ' ', b.Authorfirst) as Authorname, b.Bookname as Bookname, e.Spcode as espcode, e.Spauthor, e.Bookcode, e.Year, e.Bookname, e.Auname, DATE_FORMAT(e.Date, '%d/%m/%Y') as Date, e.Tim, e.Who, e.Copies, e.Papers, e.S1, e.S2, e.S3, e.S4, e.S5, e.S6, e.S7, e.S8, e.S9, e.Sum, e.Mem1  from  books.book b, books.expences e WHERE e.Date BETWEEN '" + startDate + "' and '" + endDate + "' and b.Authorlast like '%" + authorlast + "%' and b.Bookname like '%" + bookname + "%' and b.Spcode = e.Bookcode prder by e.Bookcode desc; ";
                Query = "SELECT  CONCAT( b.Authorlast, ' ', b.Authorfirst) as Authorname, b.Bookname as Bookname, e.Spcode as espcode, e.Bookcode, e.Year, DATE_FORMAT(e.Date, '%d/%m/%Y') as Date, e.Tim, e.Who, e.Copies, e.Papers, e.S1, e.S2, e.S3, e.S4, e.S5, e.S6, e.S7, e.S8, e.S9, e.Sum, e.Mem1  from  books.book b, books.expences e WHERE e.Date BETWEEN '" + startDate + "' and '" + endDate + "' and b.Authorlast like '%" + authorlast + "%' and b.Bookname like '%" + bookname + "%' and b.Spcode = e.Bookcode prder by e.Bookcode desc; ";
            }
            MySqlCommand order_data = new MySqlCommand(Query, this.sqlConn);

            try
            {
                order_data.ExecuteNonQuery();

                MySqlDataReader myReader = order_data.ExecuteReader();

                DataReaderAdapter dar = new DataReaderAdapter();
                dar.FillFromReader(returnValue, myReader);

                myReader.Close();
            }
            catch (Exception excp)
            {
                Exception myExcp = new Exception("Could Retrieve the User List " +
                    excp.Message, excp);
                throw (myExcp);
            }
            //string previousEntry = "";
            //string newEntry = "";
            //foreach (DataRow dr in returnValue.Rows)
            //{
            //    newEntry = dr["espcode"].ToString();
            //    MessageBox.Show(dr["Bookname"].ToString());
            //    if (String.Compare(previousEntry, newEntry) == 0 )
            //    {
            //        DataColumn[] myPrimaryKey = new DataColumn[1];
            //        myPrimaryKey[0] = added.Columns["espcode"];
            //        added.PrimaryKey = myPrimaryKey;
            //        DataRow myEditDataRow = added.Rows.Find(newEntry.ToString());
            //        foreach (DataColumn dc in myEditDataRow.Table.Columns)
            //        {
            //            MessageBox.Show("ok");
            //        }
            //    }
            //    else
            //    {
            //        previousEntry = dr["espcode"].ToString();
            //        added.ImportRow(dr);
            //        MessageBox.Show("added row");
            //    }

           // }
            return returnValue;
        }

        public DataTable book_details(int bookcode)
        {
            DataTable returnValue = new DataTable();
            string Query = "SELECT  b.Authorlast, b.Authorfirst, b.Bookname, e.Spcode as espcode, e.Spauthor, e.Bookcode, e.Year, e.Bookname, e.Auname, DATE_FORMAT(e.Date, '%d/%m/%Y') as Date, e.Tim, e.Who, e.Copies, e.Papers, e.S1, e.S2, e.S3, e.S4, e.S5, e.S6, e.S7, e.S8, e.S9, e.Sum, e.Mem1  from  books.book b, books.expences e where b.Spcode = e.Bookcode and e.Bookcode = '" + bookcode + "'  order by e.Date asc  ;";

            MySqlCommand order_data = new MySqlCommand(Query, this.sqlConn);

            try
            {
                order_data.ExecuteNonQuery();

                MySqlDataReader myReader = order_data.ExecuteReader();

                DataReaderAdapter dar = new DataReaderAdapter();
                dar.FillFromReader(returnValue, myReader);

                myReader.Close();
            }
            catch (Exception excp)
            {
                Exception myExcp = new Exception("Could Retrieve the User List " +
                    excp.Message, excp);
                throw (myExcp);
            }

            return returnValue;
        }

        public DataTable export_books(int bookcode)
        {
            DataTable returnValue = new DataTable();
            string query = "SELECT  b.Authorlast, b.Authorfirst, b.Bookname, e.Spcode as espcode, e.Spauthor, e.Bookcode, e.Year, e.Bookname, e.Auname, DATE(e.Date), e.Tim, e.Who, e.Copies, e.Papers, e.S1, e.S2, e.S3, e.S4, e.S5, e.S6, e.S7, e.S8, e.S9, e.Sum, e.Mem1  from  books.book b, books.expences e where b.Spcode = e.Bookcode and e.Bookcode = '" + bookcode + "'    ;";
            string query2 = "SELECT CONCAT( b.Authorlast, ' ', b.Authorfirst) as name, b.Bookname, DATE_FORMAT(e.Date, '%d/%m/%Y') as EntryDate, e.Tim , e.Who , e.Copies , e.Papers , e.S1 , e.S2 , e.S3, e.S4, e.S5, e.S6, e.S7, e.S8, e.S9, e.Sum, e.Mem1  from  books.book b, books.expences e where b.Spcode = e.Bookcode and e.Bookcode = '" + bookcode + "'    ;";
            MySqlCommand order_data = new MySqlCommand(query2, this.sqlConn);

            try
            {
                order_data.ExecuteNonQuery();

                MySqlDataReader myReader = order_data.ExecuteReader();

                DataReaderAdapter dar = new DataReaderAdapter();
                dar.FillFromReader(returnValue, myReader);

                myReader.Close();
            }
            catch (Exception excp)
            {
                Exception myExcp = new Exception("Could Retrieve the User List " +
                    excp.Message, excp);
                throw (myExcp);
            }
            return returnValue;
        }


        public DataTable book_transaction(int spcode, string role)
        {
            DataTable returnValue = new DataTable();
            string Query;
            if (role == "update")
            {
                Query = "SELECT  b.Authorlast, b.Authorfirst, b.Bookname,   e.Spcode, e.Spauthor, e.Bookcode, e.Year, e.Bookname, e.Auname, e.Date, e.Tim, e.Who, e.Copies, e.Papers, e.S1, e.S2, e.S3, e.S4, e.S5, e.S6, e.S7, e.S8, e.S9, e.Sum, e.Mem1  from    books.book b, books.expences e where b.spcode = e.bookcode and e.spcode = '" + spcode + "'    ;";
            }
            else
            {
                Query = "SELECT  b.Authorlast as Authorlast, b.Authorfirst, b.Bookname as Bookname,   e.Spcode, e.Spauthor, e.Bookcode, e.Year, e.Bookname, e.Auname, e.Date, e.Tim, e.Who, e.Copies, e.Papers, e.S1, e.S2, e.S3, e.S4, e.S5, e.S6, e.S7, e.S8, e.S9, e.Sum, e.Mem1  from    books.book b, books.expences e where b.spcode = '" + spcode + "' and e.bookcode = '" + spcode + "' OR e.spauthor = '" + spcode + "'  ;";
            }
            MySqlCommand order_data = new MySqlCommand(Query, this.sqlConn);

            try
            {
                order_data.ExecuteNonQuery();

                MySqlDataReader myReader = order_data.ExecuteReader();

                DataReaderAdapter dar = new DataReaderAdapter();
                dar.FillFromReader(returnValue, myReader);
                

                myReader.Close();
            }
            catch (Exception excp)
            {
                Exception myExcp = new Exception("Could Retrieve the User List " +
                    excp.Message, excp);
                throw (myExcp);
            }

            return returnValue;
        }


        public int AddBook(ArrayList trans_data)
        {
            //Spcode, Authorlast, Authorfirst, Bookname, Amount, Date, A1, A2, A3, A4, A5, A6, A7, A8, A9, Sum, N1, N2, N3, Mem1

            DataTable returnValue = new DataTable();

            string getMaxSpcode = "SELECT max(Spcode) FROM book";
            string Authorlast = trans_data[0].ToString().Replace("'", "''");

            string Authorfirst = trans_data[1].ToString().Replace("'", "''");
            string Bookname = trans_data[2].ToString().Replace("'", "''");
            string Spcode;
            int index;
            MySqlCommand getCode = new MySqlCommand(getMaxSpcode, this.sqlConn);
            try
            {
                getCode.ExecuteNonQuery();

                MySqlDataReader myReader = getCode.ExecuteReader();

                DataReaderAdapter dar = new DataReaderAdapter();
                dar.FillFromReader(returnValue, myReader);
                Spcode = returnValue.Rows[0][0].ToString();
                index = Int32.Parse(Spcode) + 1;

                myReader.Close();
            }
            catch (Exception excp)
            {
                Exception myExcp = new Exception("Could Retrieve the User List " +
                    excp.Message, excp);
                throw (myExcp);
            }

            string query = "INSERT INTO book( `AuthorLast`, `Authorfirst`, `Bookname`) values ( '" + Authorlast + "', '" + Authorfirst + "', '" + Bookname + "')";
            //string query = "INSERT INTO Expences(`Spcode`, `date`, `Tim`, `Who`, `Copies`, `Papers`, `S1`, `S2`, `S3`, `S4`, `S5`, `S6`, `S7`, `S8`, `S9`, `Sum`, `Auname`, `Bookname`, `Year`) values" +
            //             "('" + index + "','" + date + "','" + Tim + "' ,'" + Who + "' ,'" + Copies + "' ,'" + Papers + "' ,'" + S1 + "','" + S2 + "' ,'" + S3 + "' ,'" + S4 + "' ,'" + S5 + "' ,'" + S6 +
            //             "' ,'" + S7 + "' ,'" + S8 + "' ,'" + S9 + "' ,'" + Sum + "','" + Auname + "','" + Bookname + "','" + Year + "' )";

            MySqlCommand addBook = new MySqlCommand(query, this.sqlConn);

            try
            {
                addBook.ExecuteNonQuery();
            }
            catch (Exception excp)
            {
                Exception myExcp = new Exception("Could not Add Book. Error: " +
                    excp.Message, excp);
                throw (myExcp);
            }
            return index;
        }

        


        public void delete_book(int rowid)
        {
            string query = "DELETE b.*,e.* FROM books.book b, books.expences e where b.Spcode = " + rowid + " and e.Bookcode = " + rowid;
            int returnValue = 0;
            MySqlCommand DelCustomer = new MySqlCommand(query, this.sqlConn);

            try
            {
                DelCustomer.ExecuteNonQuery();

                MySqlDataReader myReader = DelCustomer.ExecuteReader();

                while (myReader.Read() != false)
                {
                    returnValue = myReader.GetInt32(0);
                }

                myReader.Close();
            }
            catch (Exception excp)
            {
                Exception myExcp = new Exception("Could not Delete Customer. Error: " +
                    excp.Message, excp);
                throw (myExcp);
            }

        }

        public void delete_trans(int rowid)
        {
            //string query = "DELETE b.*,e.* FROM books.book b, books.expences e where b.Spcode = " + rowid + " and e.Bookcode = " + rowid;
            string delete_expense =  "DELETE FROM `expences` WHERE `expences`.`Spcode` = "  + rowid;
            int returnValue = 0;
            MySqlCommand DelExpense = new MySqlCommand(delete_expense, this.sqlConn);

            try
            {
                DelExpense.ExecuteNonQuery();

                MySqlDataReader myReader = DelExpense.ExecuteReader();

                while (myReader.Read() != false)
                {
                    returnValue = myReader.GetInt32(0);
                }

                myReader.Close();
            }
            catch (Exception excp)
            {
                Exception myExcp = new Exception("Could not Delete Transaction. Error: " +
                    excp.Message, excp);
                throw (myExcp);
            }

        }

        public void AddTrans(ArrayList trans_data, int index)
        {
            string Tim = (string)trans_data[0];
            string Who = (string)trans_data[1];
            string Copies = (string)trans_data[2];
            string Papers = (string)trans_data[3];
            string S1 = (string)trans_data[4];
            string S2 = (string)trans_data[5];
            string S3 = (string)trans_data[6];
            string S4 = (string)trans_data[7];
            string S5 = (string)trans_data[8];
            string S6 = (string)trans_data[9];
            string S7 = (string)trans_data[10];
            string S8 = (string)trans_data[11];
            string S9 = (string)trans_data[12];
            string Sum = (string)trans_data[13];
         //   DateTime date = (DateTime)trans_data[14];
            string date = (string)trans_data[14];

            string query = "INSERT INTO expences(`Spcode` , `Bookcode`, `date` , `Tim`, `Who`, `Copies`, `Papers`, `S1`, `S2`, `S3`, `S4`, `S5`, `S6`, `S7`, `S8`, `S9`, `Sum`) values" +
                "(NULL, '" + index + "','" + date + "','" + Tim + "' ,'" + Who + "' ,'" + Copies + "' ,'" + Papers + "' ,'" + S1 + "','" + S2 + "' ,'" + S3 + "' ,'" + S4 + "' ,'" + S5 + "' ,'" + S6 +
                             "' ,'" + S7 + "' ,'" + S8 + "' ,'" + S9 + "' ,'" + Sum + "')";

            MySqlCommand add_trans = new MySqlCommand(query, this.sqlConn);

            try
            {
                add_trans.ExecuteNonQuery();
            }
            catch (Exception excp)
            {
                Exception myExcp = new Exception("Could not Add Book. Error: " +
                    excp.Message, excp);
                throw (myExcp);
            }

        }



        public void UpdateTrans(ArrayList trans_data, int row)
        {


            //   ArrayList cust_data2 = new ArrayList();

            string Tim = (string)trans_data[0];
            string Who = (string)trans_data[1];
            string Copies = (string)trans_data[2];
            string Papers = (string)trans_data[3];
            string S1 = (string)trans_data[4];
            string S2 = (string)trans_data[5];
            string S3 = (string)trans_data[6];
            string S4 = (string)trans_data[7];
            string S5 = (string)trans_data[8];
            string S6 = (string)trans_data[9];
            string S7 = (string)trans_data[10];
            string S8 = (string)trans_data[11];
            string S9 = (string)trans_data[12];
            string Sum = (string)trans_data[13];
          //  DateTime date = (DateTime)trans_data[14];
            string date = (string)trans_data[14];





            string Query = "UPDATE expences SET Date = '" + date + "', Tim = '" + Tim + "', Who = '" + Who + "'," +
                "Copies = '" + Copies + "', Papers = '" + Papers + "', S1 = '" + S1 + "'," +
                "S2 = '" + S2 + "', S3 = '" + S3 + "', S4 = '" + S4 + "'," +
                "S5 = '" + S5 + "', S6 = '" + S6 + "', S7 = '" + S7 + "'," +
            "S8 = '" + S8 + "', S9 = '" + S9 + "'," +
            "Sum = '" + Sum + "'" + " WHERE spcode = '" + row + "'";



            MySqlCommand UpdateUser = new MySqlCommand(Query, this.sqlConn);

            try
            {
                UpdateUser.ExecuteNonQuery();
            }
            catch (Exception excp)
            {
                Exception myExcp = new Exception("Could not Update Book Transaction. Error: " +
                    excp.Message, excp);
                throw (myExcp);
            }
        }

        
        public void Update_Author(string field, string value, string row)
        {
            string Query = "UPDATE book SET " + field + " = '" + value + "' WHERE spcode = '" + row + "'";

            MySqlCommand UpdateUser = new MySqlCommand(Query, this.sqlConn);

            try
            {
                UpdateUser.ExecuteNonQuery();
            }
            catch (Exception excp)
            {
                Exception myExcp = new Exception("Could not Update Book Transaction. Error: " +
                    excp.Message, excp);
                throw (myExcp);
            }
        }


    }

}
