using Microsoft.Office.Interop.Excel;
using System;
using System.Collections.Generic;
using System.Data;
using System.Text;
using System.Windows.Forms;

namespace books
{
    static class FastExportingMethod
    {

        public static void ExportToExcel(DataSet dataSet, string outputPath)
        {
            // Create the Excel Application object
            ApplicationClass excelApp = new ApplicationClass();

            // Create a new Excel Workbook
            Workbook excelWorkbook = excelApp.Workbooks.Add(Type.Missing);

            int sheetIndex = 0;
            string[] columns = new string[] { ".", "", "", "", "", "", ". ", ". ", ". ", ". ", ".", ".", "/.", ".", ""};
            // Copy each DataTable
            foreach (System.Data.DataTable dt in dataSet.Tables)
            {

                // Copy the DataTable to an object array
                object[,] rawData = new object[dt.Rows.Count + 4, dt.Columns.Count];
                
                // Copy the column names to the first row of the object array
                for (int col = 0; col < 2; col++)
                {
                    if (col != 1)
                        rawData[0, col + 3] = dt.Rows[0].ItemArray[col].ToString();
                    else
                        rawData[0, col + 5] = dt.Rows[0].ItemArray[col].ToString();
                    
                }

                for (int col = 0; col < columns.Length; col++)
                {
                    rawData[1, col] = columns[col];
                }

                // Copy the values to the object array
                for (int col = 2; col < dt.Columns.Count; col++)
                {
                    for (int row = 0; row < dt.Rows.Count; row++)
                    {


                        if (col == dt.Columns.Count - 1 && (row + 2) > 2)
                        {

                            decimal col1 = Decimal.Parse(dt.Rows[row].ItemArray[col - 1].ToString());
                            decimal col2 = Decimal.Parse(dt.Rows[row - 1].ItemArray[col - 1].ToString());
                            decimal col3 = col1 + col2;

                            rawData[row + 2, col - 3] = col3;
                        }
                        else
                        {
                            rawData[row + 2, col - 2] = dt.Rows[row].ItemArray[col].ToString();

                        }
                    }
                }

                // Calculate the final column letter
                string finalColLetter = string.Empty;
                string colCharset = "ABCDEFGHIJKLMNOPQRSTUVWXYZ";
                int colCharsetLen = colCharset.Length;

                if (dt.Columns.Count > colCharsetLen)
                {
                    finalColLetter = colCharset.Substring(
                        (dt.Columns.Count - 1) / colCharsetLen - 1, 1);
                }

                finalColLetter += colCharset.Substring(
                        (dt.Columns.Count - 1) % colCharsetLen, 1);

                // Create a new Sheet
                Worksheet excelSheet = (Worksheet)excelWorkbook.Sheets.Add(
                    excelWorkbook.Sheets.get_Item(++sheetIndex),
                    Type.Missing, 1, XlSheetType.xlWorksheet);

                excelSheet.Name = dt.TableName;

                // Fast data export to Excel
                string excelRange = string.Format("A1:{0}{1}",
                    finalColLetter, dt.Rows.Count + 2);

                excelSheet.get_Range(excelRange, Type.Missing).Value2 = rawData;

                // Mark the first row as BOLD
                ((Range)excelSheet.Rows[1, Type.Missing]).Font.Bold = true;
                // Mark the second row as BOLD
                ((Range)excelSheet.Rows[2, Type.Missing]).Font.Bold = true;
            }

            // Save and Close the Workbook
            excelWorkbook.SaveAs(outputPath, XlFileFormat.xlWorkbookNormal, Type.Missing,
                Type.Missing, Type.Missing, Type.Missing, XlSaveAsAccessMode.xlExclusive,
                Type.Missing, Type.Missing, Type.Missing, Type.Missing, Type.Missing);
            excelWorkbook.Close(true, Type.Missing, Type.Missing);
            excelWorkbook = null;

            // Release the Application object
            excelApp.Quit();
            excelApp = null;

            // Collect the unreferenced objects
            GC.Collect();
            GC.WaitForPendingFinalizers();

        }

    }
}
