﻿using System;
using System.Collections;
using System.IO;
using System.Windows.Forms;
using System.Collections.Generic;
using System.Text;
using System.Windows.Forms;
using System.Net;
using System.IO;
using System.Xml;

namespace books {
    public class MDUpdate
    {
        // Updater Extensions
        // Added 31/7
        private static System.IO.FileStream GetFileStream(string pathName)
        {
            return (new System.IO.FileStream(pathName, System.IO.FileMode.Open, System.IO.FileAccess.Read, System.IO.FileShare.ReadWrite));
        }

        /// <summary>
        /// Creates an MD5 Sum
        /// </summary>
        /// <param name="pathName">The path of the file which will generate an md5hash</param>
        ///
        /// <returns></returns>
        public static string GetMD5Hash(string pathName)
        {
            string strResult = "";
            string strHashData = "";

            byte[] arrbytHashValue;
            System.IO.FileStream oFileStream = null;

            System.Security.Cryptography.MD5CryptoServiceProvider oMD5Hasher = new System.Security.Cryptography.MD5CryptoServiceProvider();

            try
            {
                oFileStream = GetFileStream(pathName);
                arrbytHashValue = oMD5Hasher.ComputeHash(oFileStream);
                oFileStream.Close();

                strHashData = System.BitConverter.ToString(arrbytHashValue);
                strHashData = strHashData.Replace("-", "");
                strResult = strHashData;
            }
            catch (System.Exception ex)
            {
                System.Windows.Forms.MessageBox.Show(ex.Message, "Error!",
                           System.Windows.Forms.MessageBoxButtons.OK,
                           System.Windows.Forms.MessageBoxIcon.Error,
                           System.Windows.Forms.MessageBoxDefaultButton.Button1);
            }

            return (strResult);
        }
        public static void make_md5xml()
        {
            
            DirectoryInfo di = new DirectoryInfo( Directory.GetCurrentDirectory() );
            FileInfo[] files = di.GetFiles( "*.*", SearchOption.AllDirectories );
            XmlTextWriter writer = new XmlTextWriter("md5.xml", new System.Text.UTF8Encoding());
            writer.Formatting = Formatting.Indented;
            writer.Indentation = 4;
            writer.WriteStartDocument();
            writer.WriteComment( "Created: " + DateTime.Now.ToString() );
            writer.WriteStartElement("hashes");
            ArrayList exclude = new ArrayList();
            exclude.Add( "md5.xml" );
            exclude.Add( "Database1.accdb" );
            exclude.Add( "HydraUpdate.Exe" );

            foreach (FileInfo file in files)
            {
                if(!exclude.Contains(file.Name.ToString())){
                    writer.WriteStartElement("hash");
                    writer.WriteElementString( "filename", file.Name.ToString());
                    writer.WriteElementString( "md5" , GetMD5Hash( file.FullName.ToString() ) );
                    writer.WriteFullEndElement();
                }

            }
            writer.WriteEndElement();
            writer.WriteEndDocument();
            writer.Flush();

        }
        /// <summary>
        /// Returns an ArrayList containing the files that have failed comparison or are missing from the local installation path
        /// </summary>
        /// <param name="Path"></param>
        /// <todo>Use as a parent class. Extend it to return either the filePaths array or the number of files</todo>
        /// <returns></returns>
        public static ArrayList CompareHashes(string installation_path, Hashtable remoteHash )
        {

            Hashtable localHash = new Hashtable();
            DirectoryInfo di = new DirectoryInfo(installation_path);
            FileInfo[] files = di.GetFiles("*.*", SearchOption.AllDirectories);
            ArrayList getList = new ArrayList();

            foreach (FileInfo file in files)
            {
                 localHash.Add(file.Name, GetMD5Hash(file.FullName.ToString()));
            }
            
            //Loop through each entry in the remoteHash
            foreach (DictionaryEntry rh in remoteHash)
            {
                //If the list of local hases contains the current key from the remote hash list
                if (localHash.ContainsKey(rh.Key))
                {

                    if (localHash[rh.Key].ToString() == rh.Value.ToString())
                    {
                        //File ok. Do nothing.
                        //MessageBox.Show("File " + rh.Key.ToString() + "is valid\n\nLocal value     " + localHash[rh.Key].ToString() + "\nRemote value " + rh.Value.ToString());
                    }
                    else {
                        //File hash missmatch
                        //MessageBox.Show("File " + rh.Key.ToString() + " hash missmatch\n\nLocal value    " + localHash[rh.Key].ToString() + "\nRemote value " + rh.Value.ToString() + "\n\n");
                        ///Add to download list
                        getList.Add(rh.Key);
                    }
                }
                else
                {
                    //Missing File
                    //MessageBox.Show("Jaguar Sales is Missing a File " + rh.Key.ToString() + "\nAdd to download list");
                    getList.Add(rh.Key);
                }
                
            }

            return getList;
        }
        /// <summary>
        /// Function that reads a remote XML file containing filename-md5 data and stores the 
        /// data in a Hashtable
        /// </summary>
        /// <param name="fileName"></param>
        /// <returns></returns>
        public static Hashtable read_remote_md5(string fileName)
        {
            Hashtable files = new Hashtable();
            
            try 
            {
                WebRequest request = WebRequest.Create(fileName);

                    
                HttpWebResponse httpWebResponse = null;
                try
                {

                    httpWebResponse = request.GetResponse() as HttpWebResponse;
                }
                catch (WebException webException)
                {
                    httpWebResponse = webException.Response as HttpWebResponse;
                }



                if (httpWebResponse.StatusCode.ToString() != "NotFound")
                {


                    System.Net.WebResponse myResponse = request.GetResponse();

                    System.IO.Stream sr = myResponse.GetResponseStream();
                    System.Xml.XmlTextReader xr = new System.Xml.XmlTextReader(sr);
                    XmlDocument doc = new XmlDocument();
                    doc.Load(xr);
                    XmlElement root = doc.DocumentElement;
                    XmlNodeList nodes = root.SelectNodes("//hashes");

                    try
                    {
                        
                        foreach (XmlNode node in nodes)
                        {
                            foreach (XmlNode n in node)
                            {
                                string name = n["filename"].InnerText;
                                string md5 = n["md5"].InnerText;
                                files.Add(name, md5.ToUpper());
                                
                            }
                        }
                    }

                    catch (Exception excp)
                    {
                        Exception myExcp = new Exception("Unknown XML structure encountered. Error: " + excp.Message, excp);
                        throw (myExcp);
                    }
                }
            }
            catch (Exception excp)
            {
                Exception myExcp = new Exception("Could find hashes file. Error: " + excp.Message, excp);
                throw (myExcp);
            }
           
            return files;
        }
        //READ LOCAL XML FILE
        public static ArrayList read_local_version(string versionFile)
        {
            System.Collections.ArrayList curversion = new ArrayList();
            if (System.IO.File.Exists(versionFile))
            {

                System.IO.StreamReader sr = new System.IO.StreamReader(versionFile);
                System.Xml.XmlTextReader xr = new System.Xml.XmlTextReader(sr);
                System.Xml.XmlDocument version = new System.Xml.XmlDocument();
                version.Load(xr);
                System.Xml.XmlNodeList versionlist = version.SelectNodes("version");
                System.Xml.XmlNode revision = versionlist.Item(0).SelectSingleNode("revision");
                System.Xml.XmlNode filename = versionlist.Item(0).SelectSingleNode("filename");
                System.Xml.XmlNode xmlfile = versionlist.Item(0).SelectSingleNode("xmlfile");

                curversion.Add(float.Parse(revision.InnerText));

                curversion.Add(filename.InnerText);
            }
            else
            {
                MessageBox.Show( "ERROR" );
                curversion.Add("error");
                curversion.Add("error");
                curversion.Add("error");
            }
            return curversion;
        }

        public static ArrayList read_remote_version(string remote_version_file)
        {
            ArrayList remote_version = new ArrayList();

            try
            {
                WebRequest request = WebRequest.Create( remote_version_file );

                HttpWebResponse httpWebResponse = null;
                try
                {

                    httpWebResponse = request.GetResponse() as HttpWebResponse;
                }
                catch (WebException webException)
                {
                    httpWebResponse = webException.Response as HttpWebResponse;
                }



                if (httpWebResponse.StatusCode.ToString() != "NotFound")
                {


                    System.Net.WebResponse myResponse = request.GetResponse();

                    System.IO.Stream sr = myResponse.GetResponseStream();
                    System.Xml.XmlTextReader xr = new System.Xml.XmlTextReader( sr );
                    XmlDocument doc = new XmlDocument();
                    doc.Load( xr );
                    XmlElement root = doc.DocumentElement;
                    XmlNodeList nodes = root.SelectNodes( "//hashes" );

                    System.Xml.XmlNodeList versionlist = doc.SelectNodes( "version" );
                    System.Xml.XmlNode revision = versionlist.Item( 0 ).SelectSingleNode( "revision" );
                    System.Xml.XmlNode filename = versionlist.Item( 0 ).SelectSingleNode( "filename" );
                    System.Xml.XmlNode xmlfile = versionlist.Item( 0 ).SelectSingleNode( "xmlfile" );

                    remote_version.Add( float.Parse( revision.InnerText ) );
                  
                    remote_version.Add( filename.InnerText );
                }
            }
            catch (Exception excp)
            {
                Exception myExcp = new Exception( "Could find hashes.xml. Error: " + excp.Message, excp );
                throw ( myExcp );
            }

            return remote_version;
        }
        
        // private void Download()
        //{

                    
        //    using (WebClient wcDownload = new WebClient())
        //    {
                

        //        try
        //        {
        //            // Create a request to the file we are downloading
        //            webRequest = (HttpWebRequest)WebRequest.Create(Program.file_req);
        //            // Set default authentication for retrieving the file
        //            webRequest.Credentials = CredentialCache.DefaultCredentials;
        //            // Retrieve the response from the server
        //            webResponse = (HttpWebResponse)webRequest.GetResponse();
        //            // Ask the server for the file size and store it
        //            Int64 fileSize = webResponse.ContentLength;
                  
        //            // Open the URL for download 
        //            strResponse = wcDownload.OpenRead(Program.file_req);
        //            // Create a new file stream where we will be saving the data (local drive)
        //            strLocal = new FileStream(Program.file_path, FileMode.Create, FileAccess.Write, FileShare.None);

        //            // It will store the current number of bytes we retrieved from the server
        //            int bytesSize = 0;
        //            // A buffer for storing and writing the data retrieved from the server
        //            byte[] downBuffer = new byte[2048];

        //            // Loop through the buffer until the buffer is empty
        //            while ((bytesSize = strResponse.Read(downBuffer, 0, downBuffer.Length)) > 0)
        //            {
        //                // Write the data from the buffer to the local hard drive
        //                strLocal.Write(downBuffer, 0, bytesSize);
        //                // Invoke the method that updates the form's label and progress bar
        //                this.Invoke(new UpdateProgessCallback(this.UpdateProgress), new object[] { strLocal.Length, fileSize });
        //            }
        //            this.Invoke(new update_success_Callback(this.update_success));
        //        }
        //        finally
        //        {
        //            // When the above code has ended, close the streams
        //            strResponse.Close();
        //            strLocal.Close();
        //            if (Program.success == "yep")
        //            {
        //                this.Invoke(new change_next_button_Callback(this.change_next_button));
        //            }

        //       //     Program.success = "yep";

                    
        //        }

        //    }
        //}
    }
}